package de.fhdw.gaming.ipspiel23.ht.domain.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Random;
import java.util.function.Supplier;

import de.fhdw.gaming.core.domain.DefaultGame;
import de.fhdw.gaming.core.domain.MoveChecker;
import de.fhdw.gaming.core.domain.ObserverFactoryProvider;
import de.fhdw.gaming.ipspiel23.ht.domain.IHTGame;
import de.fhdw.gaming.ipspiel23.ht.domain.IHTPlayer;
import de.fhdw.gaming.ipspiel23.ht.domain.IHTState;
import de.fhdw.gaming.ipspiel23.ht.moves.IHTMove;
import de.fhdw.gaming.ipspiel23.ht.moves.factory.HTDefaultMoveFactory;
import de.fhdw.gaming.ipspiel23.ht.moves.factory.IHTMoveFactory;
import de.fhdw.gaming.ipspiel23.ht.strategy.IHTStrategy;

/**
 * Represents the main game class of Heads or Tails.
 */
final class HTGame extends DefaultGame<IHTPlayer, IHTState, IHTMove, IHTStrategy> implements IHTGame {
    
    /**
     * The {@link Random} instance to choose random moves.
     */
    private static final Random RANDOM = new Random();
    
    /**
     * The {@link IHTMoveFactory} to create moves.
     */
    private final IHTMoveFactory moveFactory = new HTDefaultMoveFactory();

    /**
     * Creates a new instance of {@link HTGame}.
     *
     * @param id The id of the game.
     * @param initialState The initial state of the game.
     * @param strategies The strategies of the players.
     * @param maxComputationTimePerMove The maximum computation time per move in seconds.
     * @param moveChecker The move checker.
     * @param observerFactoryProvider The observer factory provider.
     * @throws IllegalArgumentException if the given id is negative.
     * @throws InterruptedException if the current thread is interrupted.
     */
    public HTGame(final int id, final IHTState initialState, final Map<String, IHTStrategy> strategies,
            final long maxComputationTimePerMove,
            final MoveChecker<IHTPlayer, IHTState, IHTMove> moveChecker,
            final ObserverFactoryProvider observerFactoryProvider) 
                throws IllegalArgumentException, InterruptedException {
        super(id, initialState, strategies, maxComputationTimePerMove, moveChecker, observerFactoryProvider);
    }

    @Override
    public Optional<IHTMove> chooseRandomMove(final IHTPlayer player, final IHTState state) {
        final List<Supplier<IHTMove>> moveSuppliers = new ArrayList<>(3);
        moveSuppliers.add(this.moveFactory::createHeadsMove);
        moveSuppliers.add(this.moveFactory::createTailsMove);
        moveSuppliers.add(this.moveFactory::createEdgeMove);
        return Optional.of(moveSuppliers.get(RANDOM.nextInt(moveSuppliers.size())).get());
    }

    @Override
    public String toString() {
        return String.format("%s[id=%d, %s]", this.getClass().getSimpleName(), this.getId(), this.gameToString());
    }
}

